import pytest
import mock

from mlair.reference_models.abstract_reference_model import AbstractReferenceModel
from mlair.reference_models.abstract_reference_model import AbstractReferenceB2share


class TestAbstractReferenceDataHandler:

    def test_init(self):
        assert isinstance(AbstractReferenceModel(), AbstractReferenceModel)

    def test_make_reference_available_locally(self):
        arm = AbstractReferenceModel()
        with pytest.raises(NotImplementedError):
            arm.make_reference_available_locally()

    @mock.patch("os.listdir", side_effect=[True, False, FileNotFoundError, ValueError])
    def test_is_reference_available_locally(self, mock_file):
        arm = AbstractReferenceModel()
        assert arm.is_reference_available_locally("TestFile/") == True
        assert arm.is_reference_available_locally("TestFile/") == False
        assert arm.is_reference_available_locally("TestFile/") == False
        with pytest.raises(ValueError):
            arm.is_reference_available_locally("TestFile/")


class TestAbstractReferenceb2share:

    @pytest.fixture
    def ar(self):
        b2share_hosturl = "b2share.url"
        b2share_bucket = "1111-2222-3333"
        b2share_key = "b2share_key.tar.gz"
        return AbstractReferenceB2share(b2share_hosturl, b2share_bucket, b2share_key)

    def test_inheritance(self):
        assert issubclass(AbstractReferenceB2share, AbstractReferenceModel)

    def test_init(self, ar):
        assert isinstance(ar, AbstractReferenceB2share)
        assert ar.b2share_hosturl == "b2share.url"
        assert ar.b2share_bucket == "1111-2222-3333"
        assert ar.b2share_key == "b2share_key.tar.gz"

    def test_b2share_url(self, ar):
        assert isinstance(ar.b2share_url, str)
        assert ar.b2share_url == 'b2share.url/api/files/1111-2222-3333'

    def test_bar_custom(self):
        pass

    def test_download_from_b2share(self):
        pass